//////////////////////////////////////////////////////////////////
//
//  MAIN.C
//
//  Implements functions for console demo application
//
//  Author  : Rene Bllhoff
//  Created : Oct. 2007
//
//////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//										
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html	
//
//////////////////////////////////////////////////////////////////

  #include "lib.h"


#define PRINTTYPESIZE(x) printf (#x " : %d \n\r", sizeof (x));


U8 aucInput [80], aucInput2 [80];
U8 ucPtr = 0, ucLen = 0;

U8 ucHandle = 0xff;
U16 uiKbd;
U32 ulSector;
U8 aucName [17], ucTest = 0;
T_SECTOR stTemp;




B16 bFAT16FileExists (U8 *aucFileName)
{
  FILE *stFile;
	stFile = fopen ((char *) aucFileName, "rb+");
	if (stFile != NULL)
	{
		fclose (stFile);
		return TRUE;
	}
	return FALSE;
}


void vImportFile (U8 *aucHDDSource, U8 *aucOFSDest)
{
	U8 ucHandle;
	FILE *stFile;
	U32  ulLength, ulBytesToWrite;
	fpos_t stFPos;
	U8 aucBuffer [1024];

	stFile = fopen ((char *) aucHDDSource, "rb+");
  fseek (stFile, 0, 2);
	fgetpos (stFile, &stFPos);
	ulBytesToWrite = ulLength = (U32) stFPos;
	fseek (stFile, 0, 0);
	ucHandle = ucOFSCreateFile (aucOFSDest);
  if ((ulLength > 0) && (stFile != NULL) && (ucHandle != 0xff))
	{
		while (ulLength > 1024)
		{
			fread (aucBuffer, 1024, 1, stFile);
			vOFSWriteFile (ucHandle, aucBuffer, 1024);
			ulLength -= 1024;
			printf ("%d\r", ulLength);
		}
		if (ulLength > 0)
		{
			fread (aucBuffer, ulLength, 1, stFile);
			vOFSWriteFile (ucHandle, aucBuffer, (U16) ulLength);
		}
		printf ("ready      \n\r", ulLength);
		printf ("%d bytes written\n\r", ulBytesToWrite);
	}
	else
	{
    if (ucHandle == 0xff) 
    {
      printf ("\n\rDestination File '%s' (OFS) already exists\n\r", aucOFSDest);
    }
    if (stFile == NULL) 
    {
      printf ("\n\rSource File '%s' (FAT16) doesn't exist\n\r", aucHDDSource);
    }
    
  }		
	if (stFile != NULL)		{ fclose (stFile);					}
	if (ucHandle != 0xff) { vOFSCloseFile (ucHandle); }
}


void vExportFile (U8 *aucOFSSource, U8 *aucHDDDest)
{
	U8 ucHandle;
	FILE *stFile;
	U32  ulLength, ulBytesToRead;
	U8 aucBuffer [1024];

	ucHandle = ucOFSOpenFile (aucOFSSource);
	stFile = fopen ((char*) aucHDDDest, "wb+");
	if ((ucHandle != 0xff) && (stFile != NULL))
	{
		vOFSSeekBegin (ucHandle);
		ulBytesToRead = ulLength = ulOFSFileLength (ucHandle);
		
		if (ulLength > 0)
		{
			while (ulLength > 1024)
			{
				vOFSReadFile (ucHandle, aucBuffer, 1024);
				fwrite (aucBuffer, 1024, 1, stFile);
				ulLength -= 1024;
				printf ("%d         \r", ulLength);
			}
			if (ulLength > 0)
			{
				vOFSReadFile (ucHandle, aucBuffer, (U16) ulLength);
				fwrite (aucBuffer, ulLength, 1, stFile);
			}
			printf ("ready      \n\r", ulLength);
			printf ("%d bytes read\n\r", ulBytesToRead);
		}
	}
	else
	{
    if (ucHandle == 0xff) 
    {
      printf ("\n\rSource File '%s' (OFS) doesn't exist\n\r", aucOFSSource);
    }
    if (stFile == NULL) 
    {
      printf ("\n\rError opening Destination File '%s' (FAT16)\n\r", aucHDDDest);
    }
    
  }	
	if (stFile != NULL)		{ fclose (stFile);					}
	if (ucHandle != 0xff) { vOFSCloseFile (ucHandle); }
}



void vListFile (U8 *aucOFSFile)
{
	U8 ucHandle;
	U32  ulLength;
	U8 aucBuffer [1025];

	aucBuffer [1024] = 0;

	ucHandle = ucOFSOpenFile (aucOFSFile);
  if (ucHandle != 0xff)
	{
		ulLength = ulOFSFileLength (ucHandle);
		vOFSSeekBegin (ucHandle);

		while (ulLength > 1024)
		{
			vOFSReadFile (ucHandle, aucBuffer, 1024);
			printf ("%s", aucBuffer);
			ulLength -= 1024;
		}
		if (ulLength > 0)
		{
			memset (aucBuffer, 0, 1024);
			vOFSReadFile (ucHandle, aucBuffer, (U16) ulLength);
			printf ("%s", aucBuffer);
		}
	}
	else
	{
    printf ("\n\rSource File '%s' (OFS) doesn't exist\n\r", aucOFSFile);
  }
	if (ucHandle != 0xff) { vOFSCloseFile (ucHandle); }
}


U16 readkey (void)
{
  U8 temp = getch ();
  if (temp == 0) { return getch () + 0x100; }
  return temp;
}

B16 vInputLine (void)
{
  U16 uiKbd;
  memset (aucInput, 0, 80);
  do
  {
    uiKbd = readkey ();
    if (uiKbd < 0x100)
    switch (uiKbd)
    {
      case 8 :
        if (ucPtr > 0)
        {
          printf ("\x8 \x8");
          ucPtr--;
        }
        break;
      case 13 :
        printf ("\n\r");
        ucLen = ucPtr;
        ucPtr = 0;
        break;
      case 27 :
        break;
      default :
        printf ("%c", uiKbd);
        aucInput [ucPtr++] = (U8) uiKbd;
        break;
    }
  } while ((uiKbd != 13) && (uiKbd != 0x27));
  return uiKbd == 13 ? 1 : 0;
}

void vPrintHelp (void)
{
  printf ("\n\r");
  printf ("h - shows this help site\n\r");
  printf ("i - imports file from FAT to OFS\n\r");
  printf ("e - exports file from OFS to FAT\n\r");
  printf ("k - kill file\n\r");
  printf ("l - lists file in current directory in OFS\n\r");
  printf ("c - choose folder in current directory in ofs\n\r");
  printf ("m - make new folder in current directory\n\r");
  printf ("r - select root dir\n\r");
  printf ("g - go to next (linked) sector\n\r");
  printf ("j - jumps to sector\n\r");
  printf ("+ - next sector\n\r");
  printf ("- - previous sector\n\r");
  printf ("space - shows current sector\n\r");
  printf ("esc - quits console\n\r");
  printf ("\n\r");  
}


int main(int argc, char* argv[])
{


	vOFSInitFS ();

	printf ("OFS Console\n\r");
	
	vPrintHelp ();

  ulSector = ulOFSGetRootDirSector ();

  do
  {
    uiKbd = readkey ();
    switch (uiKbd)
    {
      case 'd' :
        printf ("\n\rDirectory :\n\r");
        vOFSDir ();
        break;
      case 'm' :
        printf ("md ");
        if (vInputLine ())
        {
          if (bOFSMkDir (aucInput))
          {
            printf ("\n\rDir %s created\n\r", aucInput);
          }
        }
        break;
      case 'c' :
        printf ("cd ");
        if (vInputLine ())
        {
          if (bOFSCdDir (aucInput))
          {
            printf ("\n\rDir %s selected\n\r", aucInput);
          }          
        }
        break;
      case 'k' :
        printf ("kill ");
        if (vInputLine ())
        {
          vOFSKillFile (aucInput);
          if (ucHandle != 0xff)
          {
            printf ("\n\rFile %s killed\n\r", aucInput);
          }          
        }
				break;
      case 'i' :
        printf ("import ");
        if (vInputLine ())
        {
					if (bFAT16FileExists (aucInput))
					{
						memcpy (aucInput2, aucInput, 80);
						printf ("into ");
						if (vInputLine ())
						{
							vImportFile (aucInput2, aucInput);
						}
					}
					else
					{
            printf ("\n\rSource File '%s' doesn't exist\n\r", aucInput);
          }
        }
        break;
      case 'e' :
        printf ("export ");
        if (vInputLine ())
        {
					memcpy (aucInput2, aucInput, 80);
					printf ("into ");
					if (vInputLine ())
					{
						vExportFile (aucInput2, aucInput);
					}
        }
        break;
      case 'l' :
        printf ("list file ");
        if (vInputLine ())
        {
					vListFile (aucInput);
        }
        break;
      case 'r' :
        printf ("root dir\n\r");
        ulSector = ulOFSGetRootDirSector ();
        vOFSRootDir ();
        break;
      case 'g' :
        printf ("goto next sector\n\r");
        vOFSReadSector (&stTemp, ulSector);
        if (stTemp.stHeader.ulNextSector != OFS_LAST_SECTOR)
        {
          ulSector = stTemp.stHeader.ulNextSector;
          vOFSDebugSector (ulSector);
        }
        else
        {
          printf ("last sector\n\r");
        }
        break;
      case 'j' :
        printf ("jump ");
        if (vInputLine ())
        {
          printf ("\n\r");
          ulSector = atoi ((char *) aucInput);
          vOFSDebugSector (ulSector);
        }
        break;
      case ' ' :
        vOFSDebugSector (ulSector);
        break;
      case '+' :
        ulSector++;
        vOFSDebugSector (ulSector);
        break;
      case '-' :
        if (ulSector > 0) { ulSector--; } 
        vOFSDebugSector (ulSector);
        break;
    }


  } while (uiKbd != 27);

	return 0;
}
