//////////////////////////////////////////////////////////////////
//
//  OFS.C
//
//  Implements all functionality of OFS (Own File System)
//    - Creating/Reading/Writing/Deleting Files 
//    - Creating Selecting Folders
//    - File Handles
//
//  Author  : Rene Bllhoff
//  Created : Oct. 2007
//
//////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//										
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html	
//
//////////////////////////////////////////////////////////////////

	#include "lib.h"

const T_OFS_FILE stEmptyFile = 
{
	OFS_NO_FILE,			// 	U32 ulFileNumber;			// Datei-Nummer
	0,								// 	U32 ulLength;					// Lnge der Datei (aus Directory)
	OFS_LAST_SECTOR,	// 	U32 ulStartSector;		// Start Sektor der Datei
	OFS_LAST_SECTOR,	// 	U32 ulCurrentSector;	// aktueller Sektor (schreiben/lesen)
  OFS_LAST_SECTOR,  //  U32 ulDirOrigin;    	// zugehriger directory sektor (start-sekrtor des directories)
	OFS_LAST_SECTOR,  //  U32 ulIndex;          // aktueller Index
	0,								// 	U16  uiSectorFract;		// zeiger innerhalb des Sektors (fr Byteweises lesen/schreiben)
	0,								// 	U8 ucFileState;				// datei-status
};


T_OFS_SYSTEM_INFO stOFSInfo;

T_OFS_FILE stFiles [OFS_MAX_OPEN_FILES];
T_OFS_FILE stCurrentDir;

U32 ulLastFreeSector = OFS_LAST_SECTOR;


U32 ulOFSGetFreeFileNumber (void);
U32 ulOFSFindFreeSector (void);


////////////////////////////////////////////////////////////////////////////
//
//
//  Low Level und Initialisierung (Read/Write Sector, Create FS)
//
//
////////////////////////////////////////////////////////////////////////////

void  vOFSInitFS (void)
{
  U8 ucC;
  OFS_DEVICE_INIT ();

	if (bMemoryCreated)  // mu noch gendert werden
	{
    printf ("Creating OFS\n\r");
		vOFSCreateFS (DEFAULT_NUM_SECTORS, DEFAULT_NUM_FILE);
		printf ("ready.\n\r");
	}
	vOFSLoadFS ();
  stCurrentDir = stEmptyFile;
  stCurrentDir.ulStartSector = stOFSInfo.ulRootDirStart;
  for (ucC = 0; ucC < OFS_MAX_OPEN_FILES; ucC++)
  {
    stFiles [ucC] = stEmptyFile;
  }
}

void vOFSPrepareSector (T_SECTOR *stSector, U8 ucSectorType, U32 ulFileNumber, U32 ulIndex, U32 ulLastSector)
{
  memset ((void *) stSector, 0xff, sizeof (T_SECTOR));
	stSector->stHeader.ucType       = ucSectorType;
	stSector->stHeader.ulFileNumber = ulFileNumber;
	stSector->stHeader.ulIndex      = ulIndex;
  stSector->stHeader.ulNextSector = ulLastSector;
}

void vOFSReadSector (T_SECTOR *pstSector, U32 ulSector)
{
  OFS_DEVICE_READSECTOR ( (U8 *) pstSector, ulSector);
}

void vOFSWriteSector (T_SECTOR *pstSector, U32 ulSector)
{
  OFS_DEVICE_WRITESECTOR ( (U8 *) pstSector, ulSector);
}



void  vOFSCreateFS (U32 ulNumSectors, U32 ulMaxFileCount)
{
	U32 ulNumFFNTSectors, ulCount, ulNextSector;
	T_SECTOR stTemp;
	T_OFS_DIR_ENTRY stDirEntry;

	ulNumFFNTSectors = ((ulMaxFileCount >> 3) / SECTOR_DATA_SIZE) + 1;

	stOFSInfo.ulNumSectors		= ulNumSectors;
	stOFSInfo.ulSectorSize		= SECTOR_SIZE;
	stOFSInfo.ulMaxFileCount	= ulMaxFileCount;
	stOFSInfo.ulSystemStart		= 0;
	stOFSInfo.ulFFNTStart			= stOFSInfo.ulSystemStart + 1;
	stOFSInfo.ulRootDirStart	= stOFSInfo.ulFFNTStart		+ ulNumFFNTSectors;

	// system sektor schreiben
	vOFSPrepareSector   (&stTemp, OFS_ST_SYSTEM, 0, 0, OFS_LAST_SECTOR);
	stTemp.stSystemInfo = stOFSInfo;
  vOFSWriteSector (&stTemp, stOFSInfo.ulSystemStart);

	// FFN sektoren schreiben
	for (ulCount = 0; ulCount < ulNumFFNTSectors; ulCount ++)
	{
	  ulNextSector = (ulCount < (ulNumFFNTSectors - 1)) ? ulCount +  stOFSInfo.ulFFNTStart + 1 : OFS_LAST_SECTOR;
    vOFSPrepareSector (&stTemp, OFS_ST_FFNTABLE, 0, ulCount, ulNextSector);
    stTemp.stHeader.ulIndex       = ulCount;
    vOFSWriteSector (&stTemp, stOFSInfo.ulFFNTStart + ulCount);
	}

	// Root Directory sektor schreiben
  vOFSPrepareSector (&stTemp, OFS_ST_DIR, 0, 0, OFS_LAST_SECTOR);
	memset (&stDirEntry, 0xff, sizeof (stDirEntry));
  vOFSWriteSector (&stTemp, stOFSInfo.ulRootDirStart);

	// Filenummer fr Root-Dir erzeugen
	ulOFSGetFreeFileNumber ();
}

void  vOFSLoadFS (void)
{
	T_SECTOR stTemp;

  vOFSReadSector (&stTemp, 0);
	if (stTemp.stHeader.ucType == 0x00)
	{
		stOFSInfo = stTemp.stSystemInfo;
	}
	ulOFSFindFreeSector ();
}

U32   ulOFSGetRootDirSector (void)
{
  return stOFSInfo.ulRootDirStart;
}


void  vOFSRootDir (void)
{
  stCurrentDir.ulCurrentSector = stOFSInfo.ulRootDirStart;
}

////////////////////////////////////////////////////////////////////////////
//
//
//  OFS Low Level
//
//
////////////////////////////////////////////////////////////////////////////

U32 ulOFSFindFreeSector (void)
{
	U32 ulCount;
	T_SECTOR stTemp;

	ulCount = ulLastFreeSector == OFS_LAST_SECTOR ? stOFSInfo.ulRootDirStart : ulLastFreeSector; 

	while (ulCount < DEFAULT_NUM_SECTORS)
	{
    vOFSReadSector (&stTemp, ulCount);
		if (stTemp.stHeader.ucType == 0xff) 
		{ 
			ulLastFreeSector = ulCount;
			return ulCount;
		}
		ulCount++;
	}
	return OFS_LAST_SECTOR;		
}

U32 ulOFSReserveFreeSector (U8 ucType, U32 ulFileNumber, U32 ulFileIndex)
{
  T_SECTOR stTemp;
  U32 ulSector = ulOFSFindFreeSector ();
  if (ulSector != OFS_LAST_SECTOR)
  {
    vOFSPrepareSector (&stTemp, ucType, ulFileNumber, ulFileIndex, OFS_LAST_SECTOR);
    vOFSWriteSector (&stTemp, ulSector);
  }
  return ulSector;
}


U32 ulOFSFindFreeFileNumberSector (U32 ulIndex, T_SECTOR *pstSector)
{
	U16	uiC = 0;
	U8 ucX, ucB;
	U32 ulFileNumStart = ulIndex * OFS_NUM_FFN_SECTOR;

	while (uiC < SECTOR_DATA_SIZE)
	{
		ucX = pstSector->aucBuffer [uiC];
		if (ucX != 0x00)
		{
			for (ucB = 0; ucB < 8; ucB ++)
			{
				if ((ucX & (1 << ucB)) != 0)
				{
					ulFileNumStart += uiC * 8 + ucB;
					return ulFileNumStart;
				}
			}
		}
		else
		{
			uiC++;
		}
	}

	return OFS_NO_FILE;
}

U32 ulOFSGetFreeFileNumber (void)
{
	U32 ulFileNum, ulFFNSector = stOFSInfo.ulFFNTStart;
	U16 uiFFNByte;
	U8  ucFFNBit;
	T_SECTOR stTemp;

	ulFileNum = OFS_NO_FILE;
	do
	{
    vOFSReadSector (&stTemp, ulFFNSector);
		ulFileNum = ulOFSFindFreeFileNumberSector (stTemp.stHeader.ulIndex, &stTemp);
		if (ulFileNum == OFS_NO_FILE) 
		{ 
			ulFFNSector = stTemp.stHeader.ulNextSector; 
		}
		else
		{
			uiFFNByte = (U16) ((ulFileNum >> 3) % SECTOR_DATA_SIZE);
			ucFFNBit  = (U8 )  (ulFileNum & 0x7);
			stTemp.aucBuffer [uiFFNByte] &= ~(1 << ucFFNBit);
      vOFSWriteSector (&stTemp, ulFFNSector);
		}
	} while ((ulFileNum == OFS_NO_FILE) && (stTemp.stHeader.ulNextSector != OFS_LAST_SECTOR));
	return ulFileNum;
}

////////////////////////////////////////////////////////////////////////////
//
//
//  OFS Middle Level
//
//
////////////////////////////////////////////////////////////////////////////


T_OFS_FILE stOFSNewFile (U8 ucType)
{
	U32 ulFreeSector;
	U32 ulFreeFileNumber;
	T_OFS_FILE  stFile = stEmptyFile;
	T_SECTOR    stTemp;

	if ((ucType == OFS_ST_DIR) || (ucType == OFS_ST_FILE))
	{
		ulFreeSector			= ulOFSFindFreeSector ();
		ulFreeFileNumber	= ulOFSGetFreeFileNumber ();

		vOFSPrepareSector (&stTemp, ucType, ulFreeFileNumber, 0, OFS_LAST_SECTOR);
    vOFSWriteSector   (&stTemp, ulFreeSector);

		stFile.ulStartSector    = ulFreeSector;
		stFile.ucFileState      = 0;
		stFile.uiSectorFract    = 0;
		stFile.ulIndex          = 0;
		stFile.ulCurrentSector  = ulFreeSector;
		stFile.ulFileNumber     = ulFreeFileNumber;
    stFile.ulDirOrigin      = OFS_LAST_SECTOR;
		stFile.ulLength				  = 0;
	}

	return stFile;
}

void vOFSSeekEOF (T_OFS_FILE *pstFile)
{
  T_SECTOR stTemp;
  U32 ulSector, ulLastSector;
  if (pstFile !=  NULL)
  {
    ulLastSector = ulSector = pstFile->ulStartSector;
    while (ulSector != OFS_LAST_SECTOR)
    {
      vOFSReadSector (&stTemp, ulSector);
      ulLastSector = ulSector;
      ulSector = stTemp.stHeader.ulNextSector;
    }
    pstFile->ulCurrentSector = ulLastSector;
		pstFile->ulIndex         = stTemp.stHeader.ulIndex;
    pstFile->uiSectorFract = (U16) (pstFile->ulLength % SECTOR_DATA_SIZE);
  }
}

void vOFSSeekBegin (U8 ucHandle)
{
	if ((ucHandle < OFS_MAX_OPEN_FILES) && (stFiles [ucHandle].ulStartSector != OFS_LAST_SECTOR))
	{
		stFiles [ucHandle].ulCurrentSector	= stFiles [ucHandle].ulStartSector;
		stFiles [ucHandle].uiSectorFract		= 0;
		stFiles [ucHandle].ulIndex					= 0;
	}
}

void vOFSWriteFileBuffer (T_OFS_FILE *pstFile, U8 *pucBuffer, U16 uiLength)
{
  T_SECTOR stTemp;
  U16  uiWrite;
  U32  ulNextSector;
  U32  ulIndex;
  U32  ulPtr = 0;
  U8   ucType;

  if ((pstFile != NULL) && (pstFile->ucFileState == 0x0)) 
  {
    vOFSReadSector (&stTemp, pstFile->ulCurrentSector);
    ucType = stTemp.stHeader.ucType;
    if ((pstFile->uiSectorFract + uiLength) > SECTOR_DATA_SIZE)
    {
      while ((pstFile->uiSectorFract + uiLength) > SECTOR_DATA_SIZE)
      {
        ulIndex = stTemp.stHeader.ulIndex + 1;
        ulNextSector = ulOFSFindFreeSector ();

        uiWrite = SECTOR_DATA_SIZE - pstFile->uiSectorFract;
        memcpy ((U8 *) &(stTemp.aucBuffer [pstFile->uiSectorFract]), (U8 *) &(pucBuffer [ulPtr]), uiWrite);
        stTemp.stHeader.ulNextSector = ulNextSector;
        vOFSWriteSector (&stTemp, pstFile->ulCurrentSector);

        vOFSReadSector    (&stTemp, ulNextSector);
        vOFSPrepareSector (&stTemp, ucType, pstFile->ulFileNumber, ulIndex, OFS_LAST_SECTOR);
        vOFSWriteSector   (&stTemp, ulNextSector);

        ulPtr                   += uiWrite;
        uiLength                -= uiWrite;
        pstFile->uiSectorFract   = 0;
        pstFile->ulLength       += uiWrite;
        pstFile->ulCurrentSector = ulNextSector;
      }  
      if (uiLength > 0)
      {
        memcpy ((U8 *) &(stTemp.aucBuffer [pstFile->uiSectorFract]), &(pucBuffer [ulPtr]), uiLength);
        vOFSWriteSector (&stTemp, ulNextSector);
        pstFile->uiSectorFract   = uiLength;
        pstFile->ulLength       += uiLength;
				pstFile->ulIndex				 = ulIndex;
      }
    }
    else
    {
      memcpy ((U8 *) &(stTemp.aucBuffer [pstFile->uiSectorFract]), pucBuffer, uiLength);
      vOFSWriteSector (&stTemp, pstFile->ulCurrentSector);
      pstFile->uiSectorFract  += uiLength;
      pstFile->ulLength       += uiLength;
			pstFile->ulIndex				 = stTemp.stHeader.ulIndex;
    }
  }
}

void vOFSReadFileBuffer (T_OFS_FILE *pstFile, U8 *pucBuffer, U16 uiLength)
{
  T_SECTOR stTemp;
  U16  uiRead;
  U32  ulNextSector;
  U32  ulIndex;
  U32  ulPtr = 0;
  U8   ucType;

  if ((pstFile != NULL) && (pstFile->ucFileState == 0x0)) 
  {
    vOFSReadSector (&stTemp, pstFile->ulCurrentSector);
    ucType = stTemp.stHeader.ucType;
    if ((pstFile->uiSectorFract + uiLength) > SECTOR_DATA_SIZE)
    {
      while ((pstFile->uiSectorFract + uiLength) > SECTOR_DATA_SIZE)
      {

        uiRead = SECTOR_DATA_SIZE - pstFile->uiSectorFract;
        memcpy ((U8 *) &(pucBuffer [ulPtr]), (U8 *) &(stTemp.aucBuffer [pstFile->uiSectorFract]), uiRead);
        ulNextSector = stTemp.stHeader.ulNextSector;

        vOFSReadSector    (&stTemp, ulNextSector);

        ulPtr                   += uiRead;
        uiLength                -= uiRead;
        pstFile->uiSectorFract   = 0;
        pstFile->ulCurrentSector = ulNextSector;
      }  
      if (uiLength > 0)
      {
        memcpy ((U8 *) &(pucBuffer [ulPtr]), &(stTemp.aucBuffer [pstFile->uiSectorFract]), uiLength);

        pstFile->uiSectorFract   = uiLength;
      }
    }
    else
    {
      memcpy ((U8 *) pucBuffer, &(stTemp.aucBuffer [pstFile->uiSectorFract]), uiLength);
      pstFile->uiSectorFract  += uiLength;
    }
  }
}

void vOFSSeek (T_OFS_FILE *pstFile, U32 ulPosition)
{
  if ((pstFile != NULL) && (pstFile->ulLength < ulPosition))
  {
    
  }
}


void vOFSSetDirEntry (T_SECTOR *pstTemp, U8 ucEntry, T_OFS_FILE stFile, U8 *pucName, B16 bFileIsDir)
{
  T_OFS_DIR_ENTRY stDir;
  if ((pstTemp != NULL) && (ucEntry < OFS_NUM_DIR_ENTRIES_SECTOR))
  {
    strcpy (stDir.acName, pucName);
    stDir.ucType        = bFileIsDir ? OFS_ST_DIR : OFS_ST_FILE;
    stDir.ucAttr        = 0;
    stDir.ulFileNumber  = stFile.ulFileNumber;
    stDir.ulSize        = stFile.ulLength;
    stDir.ulSectorStart = stFile.ulStartSector;
    stDir.uiDummy       = 0;
    pstTemp->astDirEntries [ucEntry] = stDir;
  }
}

void vOFSAppendFileToDirectory (U32 ulStartDirSector, T_OFS_FILE *pstFile, U8 *pucName, B16 bFileIsDir)
{
  T_SECTOR stTemp;
  U32 ulNextSector, ulLastDirSector;
  U32 ulIndex;
  U8  ucC;

  do
  {
    vOFSReadSector (&stTemp, ulStartDirSector);
    for (ucC = 0; ucC < OFS_NUM_DIR_ENTRIES_SECTOR; ucC ++)
    {
      if (stTemp.astDirEntries [ucC].ucType == 0xff)
      {
        vOFSSetDirEntry (&stTemp, ucC, *pstFile, pucName, bFileIsDir);
        vOFSWriteSector (&stTemp, ulStartDirSector);
        return;
      }
    }
    ulLastDirSector   = ulStartDirSector;
    ulStartDirSector  = stTemp.stHeader.ulNextSector;
    
  } while (stTemp.stHeader.ulNextSector != OFS_LAST_SECTOR);
  ulIndex         = stTemp.stHeader.ulIndex + 1;
  ulNextSector    = ulOFSFindFreeSector ();
  stTemp.stHeader.ulNextSector = ulNextSector;
  vOFSWriteSector (&stTemp, ulLastDirSector);
  if (ulNextSector != OFS_LAST_SECTOR)
  {
    vOFSPrepareSector (&stTemp, OFS_ST_DIR, pstFile->ulFileNumber, ulIndex, OFS_LAST_SECTOR);
    vOFSSetDirEntry   (&stTemp, 0, *pstFile, pucName, bFileIsDir);
    vOFSWriteSector   (&stTemp, ulNextSector);
  }
}

void vOFSUpdateFileLengthInDir (T_OFS_FILE *pstFile)
{
  T_SECTOR stTemp;
  U32 ulStartDirSector;
  U8  ucC;

  ulStartDirSector = pstFile->ulDirOrigin == OFS_LAST_SECTOR ? stOFSInfo.ulRootDirStart : pstFile->ulDirOrigin;

  do
  {
    vOFSReadSector (&stTemp, ulStartDirSector);
    for (ucC = 0; ucC < OFS_NUM_DIR_ENTRIES_SECTOR; ucC ++)
    {
      if (stTemp.astDirEntries [ucC].ulFileNumber == pstFile->ulFileNumber)
      {
        stTemp.astDirEntries [ucC].ulSize = pstFile->ulLength;
        vOFSWriteSector (&stTemp, ulStartDirSector);
        return;
      }
    }
    ulStartDirSector  = stTemp.stHeader.ulNextSector;
    
  } while (stTemp.stHeader.ulNextSector != OFS_LAST_SECTOR);
}

void vOFSRemoveFileInDir (T_OFS_FILE *pstFile)
{
  T_SECTOR stTemp;
  U32 ulStartDirSector;
  U8  ucC;

  ulStartDirSector = pstFile->ulDirOrigin == OFS_LAST_SECTOR ? stOFSInfo.ulRootDirStart : pstFile->ulDirOrigin;

  do
  {
    vOFSReadSector (&stTemp, ulStartDirSector);
    for (ucC = 0; ucC < OFS_NUM_DIR_ENTRIES_SECTOR; ucC ++)
    {

      if (stTemp.astDirEntries [ucC].ulFileNumber == pstFile->ulFileNumber)
      {
				memset (stTemp.astDirEntries [ucC].acName, 0xff, sizeof (stTemp.astDirEntries [ucC].acName));
				stTemp.astDirEntries [ucC].ucAttr = 0xff;
				stTemp.astDirEntries [ucC].ucType = 0xff;
				stTemp.astDirEntries [ucC].uiDummy = 0xff;
				stTemp.astDirEntries [ucC].ulFileNumber = OFS_NO_FILE;
				stTemp.astDirEntries [ucC].ulSectorStart = OFS_LAST_SECTOR;
				stTemp.astDirEntries [ucC].ulSize = OFS_NO_FILE;
        vOFSWriteSector (&stTemp, ulStartDirSector);
        return;
      }
    }
    ulStartDirSector  = stTemp.stHeader.ulNextSector;
    
  } while (stTemp.stHeader.ulNextSector != OFS_LAST_SECTOR);
}


T_OFS_FILE stOFSFindFile (U32 ulStartDirSector, U8 *pucName, B16 bFileIsDir)
{
	U32 ulDirSector = ulStartDirSector, ulDirOrg = ulStartDirSector;
	U8  ucC, ucType = bFileIsDir ? OFS_ST_DIR : OFS_ST_FILE;
	T_SECTOR stTemp;
  T_OFS_FILE stResult = stEmptyFile;

	do
	{
    vOFSReadSector (&stTemp, ulDirSector);
		for (ucC = 0; ucC < OFS_NUM_DIR_ENTRIES_SECTOR; ucC ++)
		{
			if ((strcmp (stTemp.astDirEntries [ucC].acName, pucName) == 0) && (stTemp.astDirEntries [ucC].ucType == ucType))
			{
			  stResult.ulFileNumber			= stTemp.astDirEntries [ucC].ulFileNumber;
			  stResult.uiSectorFract		= 0;
			  stResult.ulCurrentSector	= stTemp.astDirEntries [ucC].ulSectorStart;
			  stResult.ulStartSector		=	stTemp.astDirEntries [ucC].ulSectorStart;
			  stResult.ucFileState			= 0;
				stResult.ulIndex					= 0;
        stResult.ulDirOrigin      = ulDirOrg;
			  stResult.ulLength					= stTemp.astDirEntries [ucC].ulSize;
				return stResult;
			}
		}
		ulDirSector = stTemp.stHeader.ulNextSector;
	}	while (stTemp.stHeader.ulNextSector != OFS_LAST_SECTOR);

	return stResult;
}

////////////////////////////////////////////////////////////////////////////
//
//
//
//
//
////////////////////////////////////////////////////////////////////////////

U8 ucOFSFindFreeFileEntry (void)
{
  U8 ucC = 0;
  do
  {
    if (stFiles [ucC].ulFileNumber == OFS_NO_FILE)
    {
      return ucC;
    }
    else
    {
      ucC++;
    }
  } while (ucC < OFS_MAX_OPEN_FILES);
  return 0xff;
}

U8 stParseNextPathPart (U8 *pucCopyTo, U8* pucPath, U16 *puiPPtr)  // 00 = ok file, 01 = ok dir 02 = err
{
  U8 ucC;

  for (ucC = 0; ucC < 17; ucC ++) { pucCopyTo [ucC] = 0; }
  ucC = 0;
  do
  {
    if (pucPath [*puiPPtr++] == '\\')   return 1;
    else if (pucPath [*puiPPtr++] == 0) return 0;
    else pucCopyTo [ucC++] = pucPath [(*puiPPtr)++];
  } while (ucC < 17);
  return 2;
}

T_OFS_FILE stParsePath (U32 ulStartSector, U8 *pucPath)
{
  T_OFS_FILE stRunningDir;
  U8  ucC, ucL, aucPart [17];
  U16 uiPtr = 0;
  

  if (ulStartSector == OFS_NO_FILE)
  {
    stRunningDir.ulStartSector = stOFSInfo.ulRootDirStart;
  }
  else
  {
    stRunningDir.ulStartSector = ulStartSector;
  }

	ucL = 0;
  do
  {
    ucC = stParseNextPathPart (aucPart, pucPath, &uiPtr);
    if (ucC == 0)
    {
      stRunningDir = stOFSFindFile (stRunningDir.ulStartSector, aucPart, FALSE);
      return stRunningDir;
    }
    else if ((ucC == 1) && (ucL > 0))
    {
      stRunningDir = stOFSFindFile (stRunningDir.ulStartSector, aucPart, TRUE);
      if (stRunningDir.ulStartSector == OFS_LAST_SECTOR) { ucC = 2; }
    }
		ucL++;
  } while (ucC != 2);
  return stEmptyFile;
}


U8 ucOFSOpenFile (U8 *pucName)
{
  U8          ucFileNo = ucOFSFindFreeFileEntry ();
  
  if (ucFileNo != 0xff)
  {
    stFiles [ucFileNo] = stOFSFindFile (stCurrentDir.ulStartSector, pucName, FALSE);
    
    if (stFiles [ucFileNo].ulStartSector == OFS_LAST_SECTOR) 
    { 
      return 0xff;  
    }
    else
    {    
      vOFSSeekEOF (&(stFiles [ucFileNo]));
      return ucFileNo;
    }
  }
  return 0xff;                                              
}


U8 ucOFSCreateFile (U8 *pucName)
{
  U8 ucFileNo;
  T_OFS_FILE stFile;
  U32 ulStartSector;

  ulStartSector = stCurrentDir.ulStartSector;

  stFile   = stOFSFindFile (ulStartSector, pucName, FALSE);
  ucFileNo = ucOFSFindFreeFileEntry ();
  
  if (stFile.ulStartSector == OFS_LAST_SECTOR)
  {
    stFile = stOFSNewFile (OFS_ST_FILE);
    
    stFiles [ucFileNo] = stFile;
    
    if (stFiles [ucFileNo].ulStartSector == OFS_LAST_SECTOR) 
    { 
      return 0xff;  
    }
    else
    {
      vOFSAppendFileToDirectory (ulStartSector, &stFile, pucName, FALSE);
      stFiles [ucFileNo].ulDirOrigin = ulStartSector;
      return ucFileNo;
    }
  }
  return 0xff;                                              
}

void vOFSWriteFile (U8 ucHandle, U8 *pucBuffer, U16 uiLength)
{
  if ((ucHandle < OFS_MAX_OPEN_FILES) && (stFiles [ucHandle].ulStartSector != OFS_LAST_SECTOR))
  {
    vOFSWriteFileBuffer (&(stFiles [ucHandle]), pucBuffer, uiLength);
  }
}

void vOFSReadFile (U8 ucHandle, U8 *pucBuffer, U16 uiLength)
{
  if ((ucHandle < OFS_MAX_OPEN_FILES) && (stFiles [ucHandle].ulStartSector != OFS_LAST_SECTOR))
  {
    vOFSReadFileBuffer (&(stFiles [ucHandle]), pucBuffer, uiLength);
  }
}

void vOFSCloseFile (U8 ucHandle)
{
  if ((ucHandle < OFS_MAX_OPEN_FILES) && (stFiles [ucHandle].ulStartSector != OFS_LAST_SECTOR))
  {
    vOFSUpdateFileLengthInDir (&stFiles [ucHandle]);
    stFiles [ucHandle] = stEmptyFile;
  }
}

U32 ulOFSFileLength (U8 ucHandle)
{
	T_OFS_FILE stTest;
	if (ucHandle < OFS_MAX_OPEN_FILES) 
	{
		if (stFiles [ucHandle].ulStartSector != OFS_LAST_SECTOR)
		{
			return stFiles [ucHandle].ulLength;
		}
	}
	return 0;
}

void vOFSKillFile (U8 *pucName)
{
	T_OFS_FILE stFile;
	U32 ulStartSector = stCurrentDir.ulStartSector == OFS_LAST_SECTOR ? stOFSInfo.ulSystemStart : stCurrentDir.ulStartSector;
	U32 ulNextSector = 0;
	T_SECTOR stTemp;

	stFile = stOFSFindFile (stCurrentDir.ulStartSector, pucName, FALSE);
	if (stFile.ulStartSector != OFS_LAST_SECTOR)
	{
	  vOFSRemoveFileInDir (&stFile);
		ulStartSector = stFile.ulStartSector;
		while (ulNextSector != OFS_LAST_SECTOR)
		{
			vOFSReadSector (&stTemp, ulStartSector);
			ulNextSector = stTemp.stHeader.ulNextSector;
			vOFSPrepareSector (&stTemp, 0xff, OFS_NO_FILE, OFS_LAST_SECTOR, OFS_LAST_SECTOR);
			vOFSWriteSector (&stTemp, ulStartSector);
			ulStartSector = ulNextSector;
		}
	}
}

B16 bOFSMkDir (U8 *pucName)
{
  T_OFS_FILE stFile, stTest;

  stTest = stOFSFindFile (stCurrentDir.ulStartSector, pucName, TRUE);
  if (stTest.ulStartSector == OFS_LAST_SECTOR)
  {
    stFile = stOFSNewFile (OFS_ST_DIR);
    vOFSAppendFileToDirectory (stCurrentDir.ulStartSector, &stFile, pucName, TRUE);
    vOFSAppendFileToDirectory (stFile.ulStartSector, &stCurrentDir, "..", TRUE);
    return TRUE; 
  }
  return FALSE;
}


B16 bOFSCdDir (U8 *pucName)
{
  T_OFS_FILE stFile;

  stFile = stOFSFindFile (stCurrentDir.ulStartSector, pucName, TRUE);
  if (stFile.ulStartSector != OFS_LAST_SECTOR) { stCurrentDir = stFile; return TRUE; }
  return FALSE; 
}



void vOutputString (U8 *pucStr, U8 ucLen)
{
  U8 ucCh;  
  while (ucLen > 0)
  {
    ucCh = *pucStr;
    if (ucCh == 0) { printf (" "); }
              else { printf ("%c", ucCh); pucStr++; }
    ucLen--;
  }
}


void vOFSDir (void)
{
  T_SECTOR stTemp;
  T_OFS_DIR_ENTRY stDir;
  U32 ulSector = stCurrentDir.ulStartSector;
  U8 ucC;
  
  printf (" File Name         Leng   Start \n\r");
  printf ("--------------------------------\n\r");

  do
  {
    vOFSReadSector (&stTemp, ulSector);
    for (ucC = 0; ucC < OFS_NUM_DIR_ENTRIES_SECTOR; ucC ++)
    {
      stDir = stTemp.astDirEntries [ucC];
      if (stDir.ucType != 0xff)
      {
        printf (" ");
        vOutputString (stDir.acName, 16);
        printf (" ");

        switch (stDir.ucType)
        {
          case OFS_ST_DIR   : printf (" <DIR> "); break;
          case OFS_ST_FILE  : printf (" %5d ", stDir.ulSize); break;
        }
        printf (" %5d ", stDir.ulSectorStart); 
        printf ("\n\r");
      }
    }
    ulSector = stTemp.stHeader.ulNextSector;
  } while (ulSector != OFS_LAST_SECTOR);
}
