//////////////////////////////////////////////////////////////////
//
//  OFS.H
//
//  Header functions of OFS (Own File System)
//
//  Author  : Rene Bllhoff
//  Created : Oct. 2007
//
//////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//										
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html	
//
//////////////////////////////////////////////////////////////////

	#ifndef __OFS_H__
		#define __OFS_H__


			//
			// 512 Byte Sektor
			//  16 Byte OFS Header
			// 496 Byte Data



      // einige defines und konstanten

			#define SECTOR_SIZE						      512
			#define SECTOR_SIZE_SHIFT  		        9
      #define OFS_MAX_OPEN_FILES           32

			#define SECTOR_DATA_SIZE	          (SECTOR_SIZE - sizeof (T_OFS_HEADER))
			#define OFS_NUM_DIR_ENTRIES_SECTOR  ((SECTOR_DATA_SIZE) / (sizeof (T_OFS_DIR_ENTRY))) 
			#define OFS_LAST_SECTOR							(U32) (-1)
			#define OFS_NO_FILE									(U32) (-1)
			#define OFS_NUM_FFN_SECTOR					(SECTOR_DATA_SIZE << 3)


			#define OFS_DEVICE_INIT()						vOFSSimInit ()
			#define OFS_DEVICE_READSECTOR(b,s)	vOFSSimReadSector (b, s)			
			#define OFS_DEVICE_WRITESECTOR(b,s) vOFSSimWriteSector (b, s)




      // sektor daten typen

			typedef struct 
			{
				U8  ucType;						// 00 = system 01 = free file number table 02 = directory entry 03 = normal file ff = unused
				U8  ucDummy;
        U16 uiCRC;						// 16-Bit CRC (noch nicht implementiert)
				U32 ulFileNumber;			// Datei Nummer
				U32 ulIndex;					// Sektor-Index innerhalb der Datei
				U32 ulNextSector;			// nchster Sektor			
			} T_OFS_HEADER;								// 16 byte

			typedef struct
			{
				U32   ulFileNumber;		// mu <> 0 sein
				U32   ulSectorStart;	// mu <> 0 sein
				U32   ulSize;					// kann 0 sein
				U8    ucType;					// 01 = directory 02 = normal file ff = unused
				U8    ucAttr;					// bit 0 = deleted 1 = hidden
				char  acName [17];		// filename + Null Terminierung
				U8    uiDummy;				// res.
			} T_OFS_DIR_ENTRY;			    // 32 byte

			typedef struct
			{
				U32   ulSystemStart;		// sektor start des filesystems (eigentlich 0)
				U32   ulFFNTStart;			// start der datei-nummern-tabelle
				U32   ulRootDirStart;		// start des root directories
				U32   ulNumSectors;			// anzahl sektoren
				U32   ulSectorSize;			// sektoren gre (bytes)
				U32   ulMaxFileCount;		// max. anzahl an dateien 
				char  acName [16];			// volume name
			} T_OFS_SYSTEM_INFO;					// 40 byte

			typedef struct
			{
				T_OFS_HEADER        stHeader;
        union
        {
          U8                  aucBuffer [SECTOR_DATA_SIZE];
          T_OFS_SYSTEM_INFO   stSystemInfo;
          T_OFS_DIR_ENTRY     astDirEntries [SECTOR_DATA_SIZE / sizeof (T_OFS_DIR_ENTRY)];
        };
			} T_SECTOR;



      // laufzeit daten

			typedef struct
			{
				U32 ulFileNumber;			// Datei-Nummer
				U32 ulLength;					// Lnge der Datei (aus Directory)
				U32 ulStartSector;		// Start Sektor der Datei
				U32 ulCurrentSector;	// aktueller Sektor (schreiben/lesen)
				U32 ulIndex;					// aktueller Index
        U32 ulDirOrigin;    	// zugehriger directory sektor (start-sekrtor des directories)
				U16 uiSectorFract;		// zeiger innerhalb des Sektors (fr Byteweises lesen/schreiben)
				U8  ucFileState;			// datei-status
			} T_OFS_FILE;                 // 20 Byte



			typedef enum
			{
        OFS_ST_SYSTEM  = 0x00,
				OFS_ST_FFNTABLE,
				OFS_ST_DIR,
				OFS_ST_FILE,

				OFS_ST_FREE    = 0xff,
			} T_OFS_SECTOR_TYPE;




			void  vOFSInitFS (void);
			void  vOFSCreateFS (U32 ulNumSectors, U32 ulMaxFileCount);
			void  vOFSLoadFS (void);
			
			U32   ulOFSGetRootDirSector (void);
			void  vOFSRootDir (void);

      void  vOFSReadSector  (T_SECTOR *pstSector, U32 ulSector);
      void  vOFSWriteSector (T_SECTOR *pstSector, U32 ulSector);

      U8    ucOFSOpenFile   (U8 *pucName);
      U8    ucOFSCreateFile (U8 *pucName);
      void  vOFSWriteFile   (U8 ucHandle, U8 *pucBuffer, U16 uiLength);
      void  vOFSReadFile    (U8 ucHandle, U8 *pucBuffer, U16 uiLength);
      void  vOFSCloseFile   (U8 ucHandle);

      void  vOFSDir (void);
      B16   bOFSMkDir (U8 *pucName);
      B16   bOFSCdDir (U8 *pucName);
			void  vOFSKillFile (U8 *pucName);

			U32 ulOFSFileLength (U8 ucHandle);
			void vOFSSeekBegin	(U8 ucHandle);

			T_OFS_FILE stParsePath (U32 ulStartSector, U8 *pucPath);


		#endif

