//////////////////////////////////////////////////////////////////
//
//  OFS_DEBUG.C
//
//  Implements simple debugging functions 
//    - Viewing Sectors 
//
//  Author  : Rene Bllhoff
//  Created : Oct. 2007
//
//////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//										
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html	
//
//////////////////////////////////////////////////////////////////

	#include "lib.h"


void vOutputHexU8 (U8 ucVal)
{
	printf ("%c%c", NIBBLE2ASCII ( (ucVal >> 4) ), NIBBLE2ASCII ( (ucVal & 15) ) );
}

void vOutputHexU16 (U16 uiVal)
{
	vOutputHexU8 ((U8) (uiVal >> 8 ));
	vOutputHexU8 ((U8) (uiVal & 255));
}

void vOutputHexU32 (U32 ulVal)
{
	vOutputHexU16  ((U16) (ulVal >>    16 ));
	vOutputHexU16  ((U16) (ulVal &  0xffff));
}

void vOFSDebugSectorContent (T_SECTOR *pstSector)
{
	U16 uiC;
	U8 aucASCIILine [17];

	if (pstSector != NULL)
	{
		aucASCIILine [16] = 0;
		printf ("Sector Type   : ");
		switch (pstSector->stHeader.ucType)
		{
			case OFS_ST_SYSTEM		: printf ("System Info");	break;
			case OFS_ST_FFNTABLE	: printf ("FFN Table  ");	break;
			case OFS_ST_DIR				: printf ("Directory  "); break;
			case OFS_ST_FILE			: printf ("File       "); break;
			case OFS_ST_FREE			: printf ("Free Sector"); break;
		}
		printf ("\n\r");
		printf ("File Number   : ");
		printf ("%5d\n\r", pstSector->stHeader.ulFileNumber);
		printf ("Index Sector  : ");
		printf ("%5d\n\r", pstSector->stHeader.ulIndex);
		printf ("Next Sector   : ");
		printf ("%5d\n\r", pstSector->stHeader.ulNextSector);


		for (uiC = 0; uiC < sizeof (pstSector->aucBuffer); uiC ++)
		{
			if ((uiC & 0xf) == 0) { vOutputHexU16 (uiC); printf ("  "); }
			aucASCIILine [uiC & 0xf] = pstSector->aucBuffer [uiC];
			if ((aucASCIILine [uiC & 0xf]) < 32) { aucASCIILine [uiC & 0xf] = '.'; }
			vOutputHexU8 (pstSector->aucBuffer [uiC]); printf (" "); 
			if ((uiC & 0xf) == 15) { printf ("%s", aucASCIILine); printf ("\n\r"); }
		}
		printf ("\n\r"); 
	}

}

void vOFSDebugSector (U32 ulSector)
{
	T_SECTOR stSector;

  vOFSReadSector (&stSector, ulSector);
  
	printf ("Sector Number : %5d\n\r", ulSector);
	vOFSDebugSectorContent (&stSector);
}



