//////////////////////////////////////////////////////////////////
//
//  OFS_SIM.C
//
//  Implements Simulation Functions for Low-Level Access 
//    - Only a simple binary File
//
//  Author  : Rene Bllhoff
//  Created : Oct. 2007
//
//////////////////////////////////////////////////////////////////	
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//										
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html	
//
//////////////////////////////////////////////////////////////////

	#include "lib.h"


	FILE *stFlashMemory;
	U32		ulNumSectors;
	U8		ucOFSMemState;    // 0 = closed 1 = open
	B16		bMemoryCreated;

	static void vOFSSimCreateMemory (U32 ulNumSectors);
	static B16 bOFSSimMemoryExists (void);


void vOFSSimInit (void)
{
	stFlashMemory = NULL;
	bMemoryCreated = FALSE;
	if (!(bOFSSimMemoryExists ()))
	{
		vOFSSimCreateMemory (DEFAULT_NUM_SECTORS);
		bMemoryCreated = TRUE;
	}
	bOFSSimOpenMemory ();
}


void vOFSSimCreateMemory (U32 ulNumSectors)
{
	U16  uiC;
	U8 aucBuffer [SECTOR_SIZE];
	U32 ulSCount;


	for (uiC = 0; uiC < sizeof (aucBuffer); uiC++)
	{
		aucBuffer [uiC] = 0xff;
	}

  stFlashMemory = fopen (FLASH_FILE_NAME, "wb+");
  if (stFlashMemory != NULL)
  {
    printf ("Creating Memory Storage\n\r");
		for (ulSCount = 0; ulSCount < ulNumSectors; ulSCount++)
		{
			fwrite (&aucBuffer, sizeof (aucBuffer), 1, stFlashMemory);
			printf ("%d kB          \r", ulSCount >> 1);
		}
    fclose (stFlashMemory);
    printf ("ready.\n\r");
  }
}

B16 bOFSSimMemoryExists (void)
{
	FILE *stFileTest;

  stFileTest = fopen (FLASH_FILE_NAME, "rb+");
  if (stFileTest != NULL)
  {
    fclose (stFileTest);
		return 1;
  }
	return 0;
}


B16 bOFSSimOpenMemory (void)
{
	U32 ulFileLength;
	fpos_t stFPos;

	ucOFSMemState = 0;
	if (bOFSSimMemoryExists ())
	{
		stFlashMemory = fopen (FLASH_FILE_NAME, "rb+");
		if (stFlashMemory != NULL)
		{
			ulFileLength = fseek (stFlashMemory, 0, 2);
			fgetpos (stFlashMemory, &stFPos);
			ulFileLength = (U32) stFPos;
			fseek (stFlashMemory, 0, 0);
			if ((ulFileLength & (SECTOR_SIZE - 1)) == 0)
			{
				ulNumSectors = ulFileLength >> SECTOR_SIZE_SHIFT;
				ucOFSMemState = 1;
				return 1;
			}
		}
	}
	return 0;
}

void vOFSSimCloseMemory (void)
{
	if (ucOFSMemState == 1)
	{
		ucOFSMemState = 0;
		fclose (stFlashMemory);
		ulNumSectors = 0;
	}
}

void vOFSSimReadSector (U8 *pucData, U32 ulSector)
{
	U32 ulPos;

  if ((ucOFSMemState != 0) && (ulSector < ulNumSectors))
	{
		ulPos = ulSector << SECTOR_SIZE_SHIFT;
		fseek (stFlashMemory, ulPos, 0);
		fread (pucData, SECTOR_SIZE, 1, stFlashMemory);		
	}
}

void vOFSSimWriteSector (U8 *pucData, U32 ulSector)
{
	U32 ulPos;

  if ((ucOFSMemState != 0) && (ulSector < ulNumSectors))
	{
		ulPos = ulSector << SECTOR_SIZE_SHIFT;
		fseek (stFlashMemory, ulPos, 0);
		fwrite (pucData, SECTOR_SIZE, 1, stFlashMemory);
	}
}



